/**
 * Copyright (c) 2008 Washington University
 */
package org.nrg.xnat.filestore;

import java.io.FileNotFoundException;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

import org.nrg.xnat.tags.TagLabel;
import org.nrg.xnat.tags.TagLabelException;
import org.nrg.xnat.tags.TagOps;

/**
 * @author Kevin A. Archie <karchie@npg.wustl.edu>
 *
 */
public class TagHierarchyURIBuilder implements URIBuilder {
    private final static String DEFAULT_FILENAME = "unnamed";
    private final FileStore store;
    private final Collection<TagLabel> tagHierarchy;
    
    
    public TagHierarchyURIBuilder(final Connection c, final FileStore store, final Iterable<String> tagHierarchy)
    throws SQLException,TagLabelException {
	this.store = store;
	this.tagHierarchy = new ArrayList<TagLabel>();
	for (final String label : tagHierarchy) {
	    this.tagHierarchy.add(new TagLabel(label));
	}
	TagOps.add(c, this.tagHierarchy);
    }
    
    
    public TagHierarchyURIBuilder(final Connection c, final FileStore store, final String...tagHierarchy)
    throws SQLException,TagLabelException {
	this(c, store, Arrays.asList(tagHierarchy));
    }
    
    
    /* (non-Javadoc)
     * @see org.nrg.xnat.filestore.URIBuilder#buildURI(javax.sql.DataSource, java.util.Map)
     */
    final public URI buildURI(final Connection c, final Map<TagLabel,Collection<String>> metadata, final String filename)
    throws SQLException {
	final StringBuilder sb = new StringBuilder();
	for (final TagLabel tag : tagHierarchy) {
	    final Collection<String> values = metadata.get(tag);
	    if (null != values) {
		final Iterator<String> vi = values.iterator();
		if (vi.hasNext()) {
		    sb.append(vi.next().replace("/", "_"));
		    sb.append("/");
		}
	    }
	}
	sb.append(null == filename || "".equals(filename) ? DEFAULT_FILENAME : filename);
	
	// Try building URIs until we find one that isn't in use.
	final String uriRoot = sb.toString();
	try {
	    URI uri = new URI(uriRoot);
	    try {
	        for (int i = 0; ; i++) {
	            store.getFile(c, uri);
	            uri = new URI(uriRoot + "_" + i);
	        }
	    } catch (FileNotFoundException e) {
	        return uri;
	    }
	} catch (URISyntaxException e) {
	    throw new RuntimeException(e);	// logic error: we shouldn't be building bad URIs
	}
    }
}
